#include <stdio.h>
#include <taginfo_c.h>
#include <stdlib.h>
#include <string.h>

#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#define TESTTITLE "test title äöüß"

int cp(const char *from, const char *to)
{
    int fd_to, fd_from;
    char buf[4096];
    ssize_t nread;
    int saved_errno;
    
    fd_from = open(from, O_RDONLY);
    if (fd_from < 0)
        return -1;
    
    fd_to = open(to, O_WRONLY | O_CREAT , 0666); //| O_EXCL
    if (fd_to < 0)
        goto out_error;
    
    while (nread = read(fd_from, buf, sizeof buf), nread > 0)
    {
        char *out_ptr = buf;
        ssize_t nwritten;
        
        do {
            nwritten = write(fd_to, out_ptr, nread);
            
            if (nwritten >= 0)
            {
                nread -= nwritten;
                out_ptr += nwritten;
            }
            else if (errno != EINTR)
            {
                goto out_error;
            }
        } while (nread > 0);
    }
    
    if (nread == 0)
    {
        if (close(fd_to) < 0)
        {
            fd_to = -1;
            goto out_error;
        }
        close(fd_from);
        
        /* Success! */
        return 0;
    }
    
  out_error:
    saved_errno = errno;
    close(fd_from);
    if (fd_to >= 0)
        close(fd_to);
    
    errno = saved_errno;
    return -1;
}


int check_list_equal(char** labels, int labels_length, char** out_labels, int out_labels_length) {
    if(labels_length != out_labels_length)
        return 0; // not equal
    int i = 0;
    for(; i < labels_length; i++) {
        const char* first = labels[i];
        const char* second = out_labels[i];

        if(strcmp(labels[i], out_labels[i]) != 0)
            return 0; // not equal
    }
    return 1; //EQUAL
}

int main(void)
{
    TagInfo_Info *info;
    
    char val[200] = TESTDIR "samples/sample.flac";
    const char* target = "/tmp/out_03.flac";
    const char* s = val;
    
    if(cp(s, target) < 0)
        return 1; //EXIT_FAILURE
    
    info = taginfo_info_factory_make(target);
    
    if(info == NULL)
        return 1; //EXIT_FAILURE
    taginfo_info_set_title(info, TESTTITLE);
    taginfo_info_set_albumartist(info, TESTTITLE "äé");
    taginfo_info_set_is_compilation(info, 1);
    taginfo_info_set_rating(info, 3);
    int labels_length = 2;
    char** labels = malloc(labels_length * sizeof(char*));
    labels[0] = strdup("label 1 äöü");
    labels[1] = strdup("label 2 äéü");
    taginfo_info_set_artist_labels(info, labels, labels_length);
    taginfo_info_set_album_labels(info, labels, labels_length);
    taginfo_info_set_track_labels(info, labels, labels_length);
    taginfo_info_write(info);
    taginfo_info_free(info);
    info = taginfo_info_factory_make(target);
    int j = 0;
    if(taginfo_info_read(info)) {
        int out_labels_length_artist = 0;
        char** out_labels_artist = taginfo_info_get_artist_labels(info, &out_labels_length_artist);
        int out_labels_length_album = 0;
        char** out_labels_album = taginfo_info_get_album_labels(info, &out_labels_length_album);
        int out_labels_length_track = 0;
        char** out_labels_track = taginfo_info_get_track_labels(info, &out_labels_length_track);
        
        if(strcmp(taginfo_info_get_title(info), TESTTITLE) == 0 &&
           check_list_equal(labels, labels_length, out_labels_artist, out_labels_length_artist) &&
           check_list_equal(labels, labels_length, out_labels_album, out_labels_length_album) &&
           check_list_equal(labels, labels_length, out_labels_track, out_labels_length_track) &&
           strcmp(taginfo_info_get_albumartist(info), (TESTTITLE "äé")) == 0 &&
           taginfo_info_get_is_compilation(info) &&
           taginfo_info_get_rating(info) == 3) {
            
            for(j = 0; j < labels_length; j++) {
                free(labels[j]);
                free(out_labels_artist[j]);
                free(out_labels_album[j]);
                free(out_labels_track[j]);
            }
            taginfo_info_free(info);
            return 0; //EXIT_SUCCESS
        }
        for(j = 0; j < labels_length; j++) {
            free(labels[j]);
            free(out_labels_artist[j]);
            free(out_labels_album[j]);
            free(out_labels_track[j]);
        }
        taginfo_info_free(info);
        return 1; //EXIT_FAILURE
    }
    else {
        taginfo_info_free(info);
        return 1; //EXIT_FAILURE
    }
}
